/**
 * Program: SerialHashImpl.java
 * Comments: A WebFlow module that writes and retrieves data to and from a 
 * serialized hashtable in the user's Descriptors directory.
 */

package WebFlow.SerialHash;

import WebFlow.*;
import java.util.*;
import java.io.*;
import java.text.*;

public class SerialHashImpl extends WebFlow.BeanContextChildSupport
    implements SerialHashOperations { 
    
    String msg_;
    org.omg.CORBA.Object peer;
    Hashtable jobHash;
    PrintWriter logOut;
    String hashname=null;    //This is set when openHash() is called.
    
    public SerialHashImpl(org.omg.CORBA.Object peer, String msg) 
	throws WebFlow.NullPointerException {
	super(peer);
	this.peer=peer;
	String msg_=msg;

	//Set up logging
	try {
	    if((new File("/tmp/monitor.log").exists())) {
		logOut=
		    new PrintWriter(new FileWriter("/tmp/monitor.log",true));
	    }
	    else {
		logOut=
		    new PrintWriter(new FileWriter("/tmp/monitor.log"));
	    }
	    
	    mylog("====================================================");
	    mylog(getTime());
	}
	catch(Exception ex) {
	    ex.printStackTrace();
	}
    }
    
    public void test() {
	System.out.println("SerialHash is up");
	mylog("SerialHash is up");
    }
    
    /**
     * Look in the user's descriptor directory, open the 
     * jobHash file there, and deserialize it.  If the file does not
     * exist, create it and serialize it.
     * The variable hashname should be the full path to the serialized
     * hashtable.
     */
    public void openHash(String hashname) {
	this.hashname=hashname;
	mylog("Job Hash opened");
	mylog("Opening "+hashname);
	boolean status=openOldFile();	
	if(!status) {
	    System.err.println("Could not create the jobHash file");
	}
	//Grok the serialized object.  
	//NOTE: Hashtables implement serializable interface.
	try{
	    ObjectInputStream in=
		new ObjectInputStream(new FileInputStream(hashname));
	    jobHash=(Hashtable)in.readObject();
	    in.close();
	}
	catch(IOException e){
	    mylog(e.toString());
	    e.printStackTrace();
	}
	catch(Exception e) {
	    e.printStackTrace();
	    mylog(e.toString());
	}
    }
    
    /**
     * Write the name/value pair.  Note we separate multiple values by
     * a '+'.
     */
    public void appendHash(String problemName,String submitParams){
	jobHash.put(problemName,submitParams);	
    }

    /**
     * Remove the entry from the hashtable.
     */
    public void deleteEntry(String jn) {
	mylog("deleteEntry entered");
	String jobName=jn;
	if(jobHash.containsKey(jobName)) {
	    Object obj=jobHash.remove(jobName);
	}
    }
    
    /**
     * Return all of the names in the hash table.
     */
    public String[] getHashNames() {
	String[] hashNames=new String[jobHash.size()];
	Enumeration keys=jobHash.keys();
	int i=0;
	while(keys.hasMoreElements()) {
	    hashNames[i]=(String)keys.nextElement();
	    i++;
	}
	return hashNames;
    }

    /**
     * Returns the values associated with a particular hash name.  This can
     * be multivalued, and so we adopt a "+" sign as the separator.  The client
     * must separate out these multiple values with a StringTokenizer.
     */
    public String[] getHashValues() {
	String[] hashValues=new String[jobHash.size()];
	Enumeration vals=jobHash.elements();
	int i=0;
	while(vals.hasMoreElements()){
	    hashValues[i]=(String)vals.nextElement();
	    i++;
	}
	return hashValues;
    }
    
    /**
     * Close the hash table and serialize it.  Note hashname is set by the
     * openHash() command.
     */
    public void closeHash(){
	mylog("closeJobhash entered");
	try {
	    ObjectOutputStream out=
		new ObjectOutputStream(new FileOutputStream(hashname));
	    out.writeObject(jobHash);
	    out.close();
	}
	catch(IOException e) {
	    e.printStackTrace();
	}
	catch(Exception e){
	    e.printStackTrace();
	}
    }
    
    /**
     * Check to see if jobHash exists.  If so, return true.
     * If jobHash file does not exist, create it, instantiate an
     * Hashtable object here, and serialize it.
     * Returns false only if the createNewFile() method fails.
     * Note that this uses hashname as set in the openHash() method.
     */
    private boolean openOldFile() {
	mylog("openOldFile entered");
	boolean status=true;
	File jh=new File(hashname);
	if(!jh.exists()) {
	    try {
		status=jh.createNewFile();
		jobHash=new Hashtable();
		closeHash();
	    }
	    catch(Exception e) {
		e.printStackTrace();
	    }
	}
	return status;
    }  
    
    /**
     * Print the argument to the log file.
     */
    private void mylog(String logline) {
	logOut.println(logline);
	logOut.flush();
    }
    
    /**
     * Return the time.
     */
    private String getTime() {
	DateFormat longDate=
	    DateFormat.getDateTimeInstance(DateFormat.FULL,DateFormat.FULL);
	String time=longDate.format(new Date());
	return time;
    }

}






